@@ -2,12 +2,11 @@
 """Example showing how to attach a read pipe to a subprocess."""
 import asyncio
 import os, sys
-from asyncio import subprocess
 
 code = """
 import os, sys
 fd = int(sys.argv[1])
-data = os.write(fd, b'data')
+os.write(fd, b'data')
 os.close(fd)
 """
 
