@@ -7,11 +7,13 @@ import io
 import os
 import pprint
 import signal
+import socket
 import stat
 import sys
+import tempfile
 import threading
 import unittest
-import unittest.mock
+from unittest import mock
 
 if sys.platform == 'win32':
     raise unittest.SkipTest('UNIX only')
@@ -23,8 +25,11 @@ from asyncio import test_utils
 from asyncio import unix_events
 
 
+MOCK_ANY = mock.ANY
+
+
 @unittest.skipUnless(signal, 'Signals are not supported')
-class SelectorEventLoopTests(unittest.TestCase):
+class SelectorEventLoopSignalTests(unittest.TestCase):
 
     def setUp(self):
         self.loop = asyncio.SelectorEventLoop()
@@ -43,14 +48,15 @@ class SelectorEventLoopTests(unittest.TestCase):
         self.loop._handle_signal(signal.NSIG + 1, ())
 
     def test_handle_signal_cancelled_handler(self):
-        h = asyncio.Handle(unittest.mock.Mock(), ())
+        h = asyncio.Handle(mock.Mock(), (),
+                           loop=mock.Mock())
         h.cancel()
         self.loop._signal_handlers[signal.NSIG + 1] = h
-        self.loop.remove_signal_handler = unittest.mock.Mock()
+        self.loop.remove_signal_handler = mock.Mock()
         self.loop._handle_signal(signal.NSIG + 1, ())
         self.loop.remove_signal_handler.assert_called_with(signal.NSIG + 1)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.unix_events.signal')
     def test_add_signal_handler_setup_error(self, m_signal):
         m_signal.NSIG = signal.NSIG
         m_signal.set_wakeup_fd.side_effect = ValueError
@@ -60,7 +66,7 @@ class SelectorEventLoopTests(unittest.TestCase):
             self.loop.add_signal_handler,
             signal.SIGINT, lambda: True)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.unix_events.signal')
     def test_add_signal_handler(self, m_signal):
         m_signal.NSIG = signal.NSIG
 
@@ -70,7 +76,7 @@ class SelectorEventLoopTests(unittest.TestCase):
         self.assertIsInstance(h, asyncio.Handle)
         self.assertEqual(h._callback, cb)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.unix_events.signal')
     def test_add_signal_handler_install_error(self, m_signal):
         m_signal.NSIG = signal.NSIG
 
@@ -88,8 +94,8 @@ class SelectorEventLoopTests(unittest.TestCase):
             self.loop.add_signal_handler,
             signal.SIGINT, lambda: True)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
-    @unittest.mock.patch('asyncio.unix_events.logger')
+    @mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.base_events.logger')
     def test_add_signal_handler_install_error2(self, m_logging, m_signal):
         m_signal.NSIG = signal.NSIG
 
@@ -105,8 +111,8 @@ class SelectorEventLoopTests(unittest.TestCase):
         self.assertFalse(m_logging.info.called)
         self.assertEqual(1, m_signal.set_wakeup_fd.call_count)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
-    @unittest.mock.patch('asyncio.unix_events.logger')
+    @mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.base_events.logger')
     def test_add_signal_handler_install_error3(self, m_logging, m_signal):
         class Err(OSError):
             errno = errno.EINVAL
@@ -120,7 +126,7 @@ class SelectorEventLoopTests(unittest.TestCase):
         self.assertFalse(m_logging.info.called)
         self.assertEqual(2, m_signal.set_wakeup_fd.call_count)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.unix_events.signal')
     def test_remove_signal_handler(self, m_signal):
         m_signal.NSIG = signal.NSIG
 
@@ -133,7 +139,7 @@ class SelectorEventLoopTests(unittest.TestCase):
         self.assertEqual(
             (signal.SIGHUP, m_signal.SIG_DFL), m_signal.signal.call_args[0])
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.unix_events.signal')
     def test_remove_signal_handler_2(self, m_signal):
         m_signal.NSIG = signal.NSIG
         m_signal.SIGINT = signal.SIGINT
@@ -150,8 +156,8 @@ class SelectorEventLoopTests(unittest.TestCase):
             (signal.SIGINT, m_signal.default_int_handler),
             m_signal.signal.call_args[0])
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
-    @unittest.mock.patch('asyncio.unix_events.logger')
+    @mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.base_events.logger')
     def test_remove_signal_handler_cleanup_error(self, m_logging, m_signal):
         m_signal.NSIG = signal.NSIG
         self.loop.add_signal_handler(signal.SIGHUP, lambda: True)
@@ -161,7 +167,7 @@ class SelectorEventLoopTests(unittest.TestCase):
         self.loop.remove_signal_handler(signal.SIGHUP)
         self.assertTrue(m_logging.info)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.unix_events.signal')
     def test_remove_signal_handler_error(self, m_signal):
         m_signal.NSIG = signal.NSIG
         self.loop.add_signal_handler(signal.SIGHUP, lambda: True)
@@ -171,7 +177,7 @@ class SelectorEventLoopTests(unittest.TestCase):
         self.assertRaises(
             OSError, self.loop.remove_signal_handler, signal.SIGHUP)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.unix_events.signal')
     def test_remove_signal_handler_error2(self, m_signal):
         m_signal.NSIG = signal.NSIG
         self.loop.add_signal_handler(signal.SIGHUP, lambda: True)
@@ -183,7 +189,7 @@ class SelectorEventLoopTests(unittest.TestCase):
         self.assertRaises(
             RuntimeError, self.loop.remove_signal_handler, signal.SIGHUP)
 
-    @unittest.mock.patch('asyncio.unix_events.signal')
+    @mock.patch('asyncio.unix_events.signal')
     def test_close(self, m_signal):
         m_signal.NSIG = signal.NSIG
 
@@ -200,21 +206,101 @@ class SelectorEventLoopTests(unittest.TestCase):
         m_signal.set_wakeup_fd.assert_called_once_with(-1)
 
 
+@unittest.skipUnless(hasattr(socket, 'AF_UNIX'),
+                     'UNIX Sockets are not supported')
+class SelectorEventLoopUnixSocketTests(unittest.TestCase):
+
+    def setUp(self):
+        self.loop = asyncio.SelectorEventLoop()
+        asyncio.set_event_loop(None)
+
+    def tearDown(self):
+        self.loop.close()
+
+    def test_create_unix_server_existing_path_sock(self):
+        with test_utils.unix_socket_path() as path:
+            sock = socket.socket(socket.AF_UNIX)
+            sock.bind(path)
+            with sock:
+                coro = self.loop.create_unix_server(lambda: None, path)
+                with self.assertRaisesRegex(OSError,
+                                            'Address.*is already in use'):
+                    self.loop.run_until_complete(coro)
+
+    def test_create_unix_server_existing_path_nonsock(self):
+        with tempfile.NamedTemporaryFile() as file:
+            coro = self.loop.create_unix_server(lambda: None, file.name)
+            with self.assertRaisesRegex(OSError,
+                                        'Address.*is already in use'):
+                self.loop.run_until_complete(coro)
+
+    def test_create_unix_server_ssl_bool(self):
+        coro = self.loop.create_unix_server(lambda: None, path='spam',
+                                            ssl=True)
+        with self.assertRaisesRegex(TypeError,
+                                    'ssl argument must be an SSLContext'):
+            self.loop.run_until_complete(coro)
+
+    def test_create_unix_server_nopath_nosock(self):
+        coro = self.loop.create_unix_server(lambda: None, path=None)
+        with self.assertRaisesRegex(ValueError,
+                                    'path was not specified, and no sock'):
+            self.loop.run_until_complete(coro)
+
+    def test_create_unix_server_path_inetsock(self):
+        sock = socket.socket()
+        with sock:
+            coro = self.loop.create_unix_server(lambda: None, path=None,
+                                                sock=sock)
+            with self.assertRaisesRegex(ValueError,
+                                        'A UNIX Domain Socket was expected'):
+                self.loop.run_until_complete(coro)
+
+    def test_create_unix_connection_path_sock(self):
+        coro = self.loop.create_unix_connection(
+            lambda: None, '/dev/null', sock=object())
+        with self.assertRaisesRegex(ValueError, 'path and sock can not be'):
+            self.loop.run_until_complete(coro)
+
+    def test_create_unix_connection_nopath_nosock(self):
+        coro = self.loop.create_unix_connection(
+            lambda: None, None)
+        with self.assertRaisesRegex(ValueError,
+                                    'no path and sock were specified'):
+            self.loop.run_until_complete(coro)
+
+    def test_create_unix_connection_nossl_serverhost(self):
+        coro = self.loop.create_unix_connection(
+            lambda: None, '/dev/null', server_hostname='spam')
+        with self.assertRaisesRegex(ValueError,
+                                    'server_hostname is only meaningful'):
+            self.loop.run_until_complete(coro)
+
+    def test_create_unix_connection_ssl_noserverhost(self):
+        coro = self.loop.create_unix_connection(
+            lambda: None, '/dev/null', ssl=True)
+
+        with self.assertRaisesRegex(
+            ValueError, 'you have to pass server_hostname when using ssl'):
+
+            self.loop.run_until_complete(coro)
+
+
 class UnixReadPipeTransportTests(unittest.TestCase):
 
     def setUp(self):
         self.loop = test_utils.TestLoop()
         self.protocol = test_utils.make_test_protocol(asyncio.Protocol)
-        self.pipe = unittest.mock.Mock(spec_set=io.RawIOBase)
+        self.pipe = mock.Mock(spec_set=io.RawIOBase)
         self.pipe.fileno.return_value = 5
 
-        fcntl_patcher = unittest.mock.patch('fcntl.fcntl')
+        fcntl_patcher = mock.patch('fcntl.fcntl')
         fcntl_patcher.start()
         self.addCleanup(fcntl_patcher.stop)
 
-        fstat_patcher = unittest.mock.patch('os.fstat')
+        fstat_patcher = mock.patch('os.fstat')
         m_fstat = fstat_patcher.start()
-        st = unittest.mock.Mock()
+        st = mock.Mock()
         st.st_mode = stat.S_IFIFO
         m_fstat.return_value = st
         self.addCleanup(fstat_patcher.stop)
@@ -233,7 +319,7 @@ class UnixReadPipeTransportTests(unittest.TestCase):
         test_utils.run_briefly(self.loop)
         self.assertIsNone(fut.result())
 
-    @unittest.mock.patch('os.read')
+    @mock.patch('os.read')
     def test__read_ready(self, m_read):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -243,7 +329,7 @@ class UnixReadPipeTransportTests(unittest.TestCase):
         m_read.assert_called_with(5, tr.max_size)
         self.protocol.data_received.assert_called_with(b'data')
 
-    @unittest.mock.patch('os.read')
+    @mock.patch('os.read')
     def test__read_ready_eof(self, m_read):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -256,7 +342,7 @@ class UnixReadPipeTransportTests(unittest.TestCase):
         self.protocol.eof_received.assert_called_with()
         self.protocol.connection_lost.assert_called_with(None)
 
-    @unittest.mock.patch('os.read')
+    @mock.patch('os.read')
     def test__read_ready_blocked(self, m_read):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -267,31 +353,35 @@ class UnixReadPipeTransportTests(unittest.TestCase):
         test_utils.run_briefly(self.loop)
         self.assertFalse(self.protocol.data_received.called)
 
-    @unittest.mock.patch('asyncio.log.logger.exception')
-    @unittest.mock.patch('os.read')
+    @mock.patch('asyncio.log.logger.error')
+    @mock.patch('os.read')
     def test__read_ready_error(self, m_read, m_logexc):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
         err = OSError()
         m_read.side_effect = err
-        tr._close = unittest.mock.Mock()
+        tr._close = mock.Mock()
         tr._read_ready()
 
         m_read.assert_called_with(5, tr.max_size)
         tr._close.assert_called_with(err)
-        m_logexc.assert_called_with('Fatal error for %s', tr)
+        m_logexc.assert_called_with(
+            test_utils.MockPattern(
+                'Fatal read error on pipe transport'
+                '\nprotocol:.*\ntransport:.*'),
+            exc_info=(OSError, MOCK_ANY, MOCK_ANY))
 
-    @unittest.mock.patch('os.read')
+    @mock.patch('os.read')
     def test_pause_reading(self, m_read):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
 
-        m = unittest.mock.Mock()
+        m = mock.Mock()
         self.loop.add_reader(5, m)
         tr.pause_reading()
         self.assertFalse(self.loop.readers)
 
-    @unittest.mock.patch('os.read')
+    @mock.patch('os.read')
     def test_resume_reading(self, m_read):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -299,26 +389,26 @@ class UnixReadPipeTransportTests(unittest.TestCase):
         tr.resume_reading()
         self.loop.assert_reader(5, tr._read_ready)
 
-    @unittest.mock.patch('os.read')
+    @mock.patch('os.read')
     def test_close(self, m_read):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
 
-        tr._close = unittest.mock.Mock()
+        tr._close = mock.Mock()
         tr.close()
         tr._close.assert_called_with(None)
 
-    @unittest.mock.patch('os.read')
+    @mock.patch('os.read')
     def test_close_already_closing(self, m_read):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
 
         tr._closing = True
-        tr._close = unittest.mock.Mock()
+        tr._close = mock.Mock()
         tr.close()
         self.assertFalse(tr._close.called)
 
-    @unittest.mock.patch('os.read')
+    @mock.patch('os.read')
     def test__close(self, m_read):
         tr = unix_events._UnixReadPipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -343,7 +433,7 @@ class UnixReadPipeTransportTests(unittest.TestCase):
         self.assertEqual(2, sys.getrefcount(self.protocol),
                          pprint.pformat(gc.get_referrers(self.protocol)))
         self.assertIsNone(tr._loop)
-        self.assertEqual(2, sys.getrefcount(self.loop),
+        self.assertEqual(4, sys.getrefcount(self.loop),
                          pprint.pformat(gc.get_referrers(self.loop)))
 
     def test__call_connection_lost_with_err(self):
@@ -356,10 +446,11 @@ class UnixReadPipeTransportTests(unittest.TestCase):
         self.pipe.close.assert_called_with()
 
         self.assertIsNone(tr._protocol)
+
         self.assertEqual(2, sys.getrefcount(self.protocol),
                          pprint.pformat(gc.get_referrers(self.protocol)))
         self.assertIsNone(tr._loop)
-        self.assertEqual(2, sys.getrefcount(self.loop),
+        self.assertEqual(4, sys.getrefcount(self.loop),
                          pprint.pformat(gc.get_referrers(self.loop)))
 
 
@@ -368,16 +459,16 @@ class UnixWritePipeTransportTests(unittest.TestCase):
     def setUp(self):
         self.loop = test_utils.TestLoop()
         self.protocol = test_utils.make_test_protocol(asyncio.BaseProtocol)
-        self.pipe = unittest.mock.Mock(spec_set=io.RawIOBase)
+        self.pipe = mock.Mock(spec_set=io.RawIOBase)
         self.pipe.fileno.return_value = 5
 
-        fcntl_patcher = unittest.mock.patch('fcntl.fcntl')
+        fcntl_patcher = mock.patch('fcntl.fcntl')
         fcntl_patcher.start()
         self.addCleanup(fcntl_patcher.stop)
 
-        fstat_patcher = unittest.mock.patch('os.fstat')
+        fstat_patcher = mock.patch('os.fstat')
         m_fstat = fstat_patcher.start()
-        st = unittest.mock.Mock()
+        st = mock.Mock()
         st.st_mode = stat.S_IFSOCK
         m_fstat.return_value = st
         self.addCleanup(fstat_patcher.stop)
@@ -402,7 +493,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
             self.loop, self.pipe, self.protocol)
         self.assertTrue(tr.can_write_eof())
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test_write(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -413,7 +504,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.assertFalse(self.loop.writers)
         self.assertEqual([], tr._buffer)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test_write_no_data(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -423,7 +514,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.assertFalse(self.loop.writers)
         self.assertEqual([], tr._buffer)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test_write_partial(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -434,7 +525,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.loop.assert_writer(5, tr._write_ready)
         self.assertEqual([b'ta'], tr._buffer)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test_write_buffer(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -446,7 +537,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.loop.assert_writer(5, tr._write_ready)
         self.assertEqual([b'previous', b'data'], tr._buffer)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test_write_again(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -457,20 +548,22 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.loop.assert_writer(5, tr._write_ready)
         self.assertEqual([b'data'], tr._buffer)
 
-    @unittest.mock.patch('asyncio.unix_events.logger')
-    @unittest.mock.patch('os.write')
+    @mock.patch('asyncio.unix_events.logger')
+    @mock.patch('os.write')
     def test_write_err(self, m_write, m_log):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
 
         err = OSError()
         m_write.side_effect = err
-        tr._fatal_error = unittest.mock.Mock()
+        tr._fatal_error = mock.Mock()
         tr.write(b'data')
         m_write.assert_called_with(5, b'data')
         self.assertFalse(self.loop.writers)
         self.assertEqual([], tr._buffer)
-        tr._fatal_error.assert_called_with(err)
+        tr._fatal_error.assert_called_with(
+                            err,
+                            'Fatal write error on pipe transport')
         self.assertEqual(1, tr._conn_lost)
 
         tr.write(b'data')
@@ -483,7 +576,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         m_log.warning.assert_called_with(
             'pipe closed by peer or os.write(pipe, data) raised exception.')
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test_write_close(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -504,7 +597,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         test_utils.run_briefly(self.loop)
         self.protocol.connection_lost.assert_called_with(None)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test__write_ready(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -516,7 +609,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.assertFalse(self.loop.writers)
         self.assertEqual([], tr._buffer)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test__write_ready_partial(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -529,7 +622,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.loop.assert_writer(5, tr._write_ready)
         self.assertEqual([b'a'], tr._buffer)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test__write_ready_again(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -542,7 +635,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.loop.assert_writer(5, tr._write_ready)
         self.assertEqual([b'data'], tr._buffer)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test__write_ready_empty(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -555,8 +648,8 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.loop.assert_writer(5, tr._write_ready)
         self.assertEqual([b'data'], tr._buffer)
 
-    @unittest.mock.patch('asyncio.log.logger.exception')
-    @unittest.mock.patch('os.write')
+    @mock.patch('asyncio.log.logger.error')
+    @mock.patch('os.write')
     def test__write_ready_err(self, m_write, m_logexc):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -570,12 +663,16 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.assertFalse(self.loop.readers)
         self.assertEqual([], tr._buffer)
         self.assertTrue(tr._closing)
-        m_logexc.assert_called_with('Fatal error for %s', tr)
+        m_logexc.assert_called_with(
+            test_utils.MockPattern(
+                'Fatal write error on pipe transport'
+                '\nprotocol:.*\ntransport:.*'),
+            exc_info=(OSError, MOCK_ANY, MOCK_ANY))
         self.assertEqual(1, tr._conn_lost)
         test_utils.run_briefly(self.loop)
         self.protocol.connection_lost.assert_called_with(err)
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test__write_ready_closing(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -592,7 +689,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.protocol.connection_lost.assert_called_with(None)
         self.pipe.close.assert_called_with()
 
-    @unittest.mock.patch('os.write')
+    @mock.patch('os.write')
     def test_abort(self, m_write):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
@@ -622,7 +719,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.assertEqual(2, sys.getrefcount(self.protocol),
                          pprint.pformat(gc.get_referrers(self.protocol)))
         self.assertIsNone(tr._loop)
-        self.assertEqual(2, sys.getrefcount(self.loop),
+        self.assertEqual(4, sys.getrefcount(self.loop),
                          pprint.pformat(gc.get_referrers(self.loop)))
 
     def test__call_connection_lost_with_err(self):
@@ -638,14 +735,14 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         self.assertEqual(2, sys.getrefcount(self.protocol),
                          pprint.pformat(gc.get_referrers(self.protocol)))
         self.assertIsNone(tr._loop)
-        self.assertEqual(2, sys.getrefcount(self.loop),
+        self.assertEqual(4, sys.getrefcount(self.loop),
                          pprint.pformat(gc.get_referrers(self.loop)))
 
     def test_close(self):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
 
-        tr.write_eof = unittest.mock.Mock()
+        tr.write_eof = mock.Mock()
         tr.close()
         tr.write_eof.assert_called_with()
 
@@ -653,7 +750,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
         tr = unix_events._UnixWritePipeTransport(
             self.loop, self.pipe, self.protocol)
 
-        tr.write_eof = unittest.mock.Mock()
+        tr.write_eof = mock.Mock()
         tr._closing = True
         tr.close()
         self.assertFalse(tr.write_eof.called)
@@ -680,7 +777,7 @@ class UnixWritePipeTransportTests(unittest.TestCase):
 class AbstractChildWatcherTests(unittest.TestCase):
 
     def test_not_implemented(self):
-        f = unittest.mock.Mock()
+        f = mock.Mock()
         watcher = asyncio.AbstractChildWatcher()
         self.assertRaises(
             NotImplementedError, watcher.add_child_handler, f, f)
@@ -699,7 +796,7 @@ class AbstractChildWatcherTests(unittest.TestCase):
 class BaseChildWatcherTests(unittest.TestCase):
 
     def test_not_implemented(self):
-        f = unittest.mock.Mock()
+        f = mock.Mock()
         watcher = unix_events.BaseChildWatcher()
         self.assertRaises(
             NotImplementedError, watcher._do_waitpid, f)
@@ -716,14 +813,14 @@ WaitPidMocks = collections.namedtuple("WaitPidMocks",
 
 class ChildWatcherTestsMixin:
 
-    ignore_warnings = unittest.mock.patch.object(log.logger, "warning")
+    ignore_warnings = mock.patch.object(log.logger, "warning")
 
     def setUp(self):
         self.loop = test_utils.TestLoop()
         self.running = False
         self.zombies = {}
 
-        with unittest.mock.patch.object(
+        with mock.patch.object(
                 self.loop, "add_signal_handler") as self.m_add_signal_handler:
             self.watcher = self.create_watcher()
             self.watcher.attach_loop(self.loop)
@@ -767,8 +864,8 @@ class ChildWatcherTestsMixin:
     def waitpid_mocks(func):
         def wrapped_func(self):
             def patch(target, wrapper):
-                return unittest.mock.patch(target, wraps=wrapper,
-                                           new_callable=unittest.mock.Mock)
+                return mock.patch(target, wraps=wrapper,
+                                  new_callable=mock.Mock)
 
             with patch('os.WTERMSIG', self.WTERMSIG) as m_WTERMSIG, \
                  patch('os.WEXITSTATUS', self.WEXITSTATUS) as m_WEXITSTATUS, \
@@ -784,7 +881,7 @@ class ChildWatcherTestsMixin:
     @waitpid_mocks
     def test_sigchld(self, m):
         # register a child
-        callback = unittest.mock.Mock()
+        callback = mock.Mock()
 
         with self.watcher:
             self.running = True
@@ -844,8 +941,8 @@ class ChildWatcherTestsMixin:
 
     @waitpid_mocks
     def test_sigchld_two_children(self, m):
-        callback1 = unittest.mock.Mock()
-        callback2 = unittest.mock.Mock()
+        callback1 = mock.Mock()
+        callback2 = mock.Mock()
 
         # register child 1
         with self.watcher:
@@ -870,7 +967,7 @@ class ChildWatcherTestsMixin:
         self.assertFalse(m.WEXITSTATUS.called)
         self.assertFalse(m.WTERMSIG.called)
 
-        # childen are running
+        # children are running
         self.watcher._sig_chld()
 
         self.assertFalse(callback1.called)
@@ -948,8 +1045,8 @@ class ChildWatcherTestsMixin:
 
     @waitpid_mocks
     def test_sigchld_two_children_terminating_together(self, m):
-        callback1 = unittest.mock.Mock()
-        callback2 = unittest.mock.Mock()
+        callback1 = mock.Mock()
+        callback2 = mock.Mock()
 
         # register child 1
         with self.watcher:
@@ -974,7 +1071,7 @@ class ChildWatcherTestsMixin:
         self.assertFalse(m.WEXITSTATUS.called)
         self.assertFalse(m.WTERMSIG.called)
 
-        # childen are running
+        # children are running
         self.watcher._sig_chld()
 
         self.assertFalse(callback1.called)
@@ -1018,7 +1115,7 @@ class ChildWatcherTestsMixin:
     @waitpid_mocks
     def test_sigchld_race_condition(self, m):
         # register a child
-        callback = unittest.mock.Mock()
+        callback = mock.Mock()
 
         with self.watcher:
             # child terminates before being registered
@@ -1039,8 +1136,8 @@ class ChildWatcherTestsMixin:
 
     @waitpid_mocks
     def test_sigchld_replace_handler(self, m):
-        callback1 = unittest.mock.Mock()
-        callback2 = unittest.mock.Mock()
+        callback1 = mock.Mock()
+        callback2 = mock.Mock()
 
         # register a child
         with self.watcher:
@@ -1092,7 +1189,7 @@ class ChildWatcherTestsMixin:
 
     @waitpid_mocks
     def test_sigchld_remove_handler(self, m):
-        callback = unittest.mock.Mock()
+        callback = mock.Mock()
 
         # register a child
         with self.watcher:
@@ -1124,7 +1221,7 @@ class ChildWatcherTestsMixin:
 
     @waitpid_mocks
     def test_sigchld_unknown_status(self, m):
-        callback = unittest.mock.Mock()
+        callback = mock.Mock()
 
         # register a child
         with self.watcher:
@@ -1161,9 +1258,9 @@ class ChildWatcherTestsMixin:
 
     @waitpid_mocks
     def test_remove_child_handler(self, m):
-        callback1 = unittest.mock.Mock()
-        callback2 = unittest.mock.Mock()
-        callback3 = unittest.mock.Mock()
+        callback1 = mock.Mock()
+        callback2 = mock.Mock()
+        callback3 = mock.Mock()
 
         # register children
         with self.watcher:
@@ -1194,7 +1291,7 @@ class ChildWatcherTestsMixin:
 
     @waitpid_mocks
     def test_sigchld_unhandled_exception(self, m):
-        callback = unittest.mock.Mock()
+        callback = mock.Mock()
 
         # register a child
         with self.watcher:
@@ -1204,16 +1301,16 @@ class ChildWatcherTestsMixin:
         # raise an exception
         m.waitpid.side_effect = ValueError
 
-        with unittest.mock.patch.object(log.logger,
-                                        "exception") as m_exception:
+        with mock.patch.object(log.logger,
+                               'error') as m_error:
 
             self.assertEqual(self.watcher._sig_chld(), None)
-            self.assertTrue(m_exception.called)
+            self.assertTrue(m_error.called)
 
     @waitpid_mocks
     def test_sigchld_child_reaped_elsewhere(self, m):
         # register a child
-        callback = unittest.mock.Mock()
+        callback = mock.Mock()
 
         with self.watcher:
             self.running = True
@@ -1238,7 +1335,6 @@ class ChildWatcherTestsMixin:
         with self.ignore_warnings:
             self.watcher._sig_chld()
 
-        callback.assert_called(m.waitpid)
         if isinstance(self.watcher, asyncio.FastChildWatcher):
             # here the FastChildWatche enters a deadlock
             # (there is no way to prevent it)
@@ -1249,8 +1345,8 @@ class ChildWatcherTestsMixin:
     @waitpid_mocks
     def test_sigchld_unknown_pid_during_registration(self, m):
         # register two children
-        callback1 = unittest.mock.Mock()
-        callback2 = unittest.mock.Mock()
+        callback1 = mock.Mock()
+        callback2 = mock.Mock()
 
         with self.ignore_warnings, self.watcher:
             self.running = True
@@ -1270,7 +1366,7 @@ class ChildWatcherTestsMixin:
     @waitpid_mocks
     def test_set_loop(self, m):
         # register a child
-        callback = unittest.mock.Mock()
+        callback = mock.Mock()
 
         with self.watcher:
             self.running = True
@@ -1279,19 +1375,16 @@ class ChildWatcherTestsMixin:
         # attach a new loop
         old_loop = self.loop
         self.loop = test_utils.TestLoop()
+        patch = mock.patch.object
 
-        with unittest.mock.patch.object(
-                old_loop,
-                "remove_signal_handler") as m_old_remove_signal_handler, \
-             unittest.mock.patch.object(
-                self.loop,
-                "add_signal_handler") as m_new_add_signal_handler:
+        with patch(old_loop, "remove_signal_handler") as m_old_remove, \
+             patch(self.loop, "add_signal_handler") as m_new_add:
 
             self.watcher.attach_loop(self.loop)
 
-            m_old_remove_signal_handler.assert_called_once_with(
+            m_old_remove.assert_called_once_with(
                 signal.SIGCHLD)
-            m_new_add_signal_handler.assert_called_once_with(
+            m_new_add.assert_called_once_with(
                 signal.SIGCHLD, self.watcher._sig_chld)
 
         # child terminates
@@ -1304,9 +1397,9 @@ class ChildWatcherTestsMixin:
     @waitpid_mocks
     def test_set_loop_race_condition(self, m):
         # register 3 children
-        callback1 = unittest.mock.Mock()
-        callback2 = unittest.mock.Mock()
-        callback3 = unittest.mock.Mock()
+        callback1 = mock.Mock()
+        callback2 = mock.Mock()
+        callback3 = mock.Mock()
 
         with self.watcher:
             self.running = True
@@ -1318,7 +1411,7 @@ class ChildWatcherTestsMixin:
         old_loop = self.loop
         self.loop = None
 
-        with unittest.mock.patch.object(
+        with mock.patch.object(
                 old_loop, "remove_signal_handler") as m_remove_signal_handler:
 
             self.watcher.attach_loop(None)
@@ -1330,7 +1423,7 @@ class ChildWatcherTestsMixin:
         self.add_zombie(61, 11)
         self.add_zombie(62, -5)
 
-        # SIGCHLD was not catched
+        # SIGCHLD was not caught
         self.assertFalse(callback1.called)
         self.assertFalse(callback2.called)
         self.assertFalse(callback3.called)
@@ -1338,7 +1431,7 @@ class ChildWatcherTestsMixin:
         # attach a new loop
         self.loop = test_utils.TestLoop()
 
-        with unittest.mock.patch.object(
+        with mock.patch.object(
                 self.loop, "add_signal_handler") as m_add_signal_handler:
 
             self.watcher.attach_loop(self.loop)
@@ -1364,8 +1457,7 @@ class ChildWatcherTestsMixin:
     @waitpid_mocks
     def test_close(self, m):
         # register two children
-        callback1 = unittest.mock.Mock()
-        callback2 = unittest.mock.Mock()
+        callback1 = mock.Mock()
 
         with self.watcher:
             self.running = True
@@ -1382,7 +1474,7 @@ class ChildWatcherTestsMixin:
             if isinstance(self.watcher, asyncio.FastChildWatcher):
                 self.assertEqual(len(self.watcher._zombies), 1)
 
-            with unittest.mock.patch.object(
+            with mock.patch.object(
                     self.loop,
                     "remove_signal_handler") as m_remove_signal_handler:
 
